from os import path 

from handle_intro.utils.helpers import get_attr
from handle_intro.infra.script import Script

from handle_intro.models.page import Section
from handle_intro.models.saved_file import SavedFile
from handle_intro.models.video import MyVideo 

from handle_intro.utils.helpers import generate_id
from handle_intro.utils.video import get_video_duration

from handle_intro.models.intro import Intro

SCRIPT_NAME = "HANDLE INTROS"
NO_DATA  = "No Sections" 
ERROR_SCRIPT = "Could not handle intros"

class HandleIntro(Script):
   
    def __init__(self, config, text_robot, logger = None):
        
        super().__init__(SCRIPT_NAME, logger)

        self.config = config
        
        self.on_lambda = get_attr(self.config, 'on_lambda', False)
        self.data = get_attr(self.config, 'data', {})
        self.temp_directory = get_attr(self.config, 'temp_directory', '/tmp')

        self.minimal_frequency = get_attr(self.config, 'minimal_frequency', 1)
        self.moderate_frequency = get_attr(self.config, 'moderate_frequency', 2)
        self.agressive_frequency = get_attr(self.config, 'agressive_frequency', 3)
        self.minimal_content_time_seconds = get_attr(self.config, 'minimal_content_time_seconds', 30)

        self.ad_mode = get_attr(self.config, 'ad_mode', 1)

        self.intro_text = get_attr(self.config, 'intro_text', '')

        self.text_robot = text_robot

        
    def replace_placeholders(self, text):
        
        user_info = self.user_config.user_info
        channel_info = self.user_config.channel_info
        new_text = text

        channel_name = get_attr(channel_info, 'name', '')
        new_text = str(new_text).replace("{{channel_name}}", f"{channel_name}")  

        return new_text   
    
   


       
    






    












    
    

         




from handle_intro.utils.json import from_json

def get_event():

        
        event = from_json('event_call.json')
        #event = from_json('event_s3.json')
        #event = from_json('event_sqs.json')
        return event



import json
import os
from types import SimpleNamespace


def from_json(file):
    f = open(file)
    data = json.load(f)
    return data

def to_json(file, data):
  os.makedirs(os.path.dirname(file), exist_ok=True)
  with open(file, 'w+', encoding='utf-8') as f:
    json.dump(data, f, ensure_ascii=False, indent=4)
   
  return data  

def dumps(data):
  
  def get_dict(data):
    try:
      return data.__dict__
    except Exception as err:
      print(err)
      pass

  return json.dumps(data, default=lambda data: get_dict(data), sort_keys=True, indent=4)

def loads(data):
  return json.loads(data, object_hook=lambda d: SimpleNamespace(**d)) 


from handle_intro.controllers.base_controller import BaseController
from handle_intro.services.base_service import BaseService

from handle_intro.models.request import Request

from handle_intro.factories.base_factory import BaseFactory

#Infra
#upload
from file_transfer.infra.s3_file_saver.s3_file_saver import S3FileSaver 
from file_transfer.infra.pure_file_saver.pure_file_saver import PureFileSaver 
from file_transfer.infra.local_file_saver.local_file_saver import LocalFileSaver 
from file_saver.infra.mongo_uploader.mongo_uploader import MongoUploader

#download
from file_transfer.infra.s3_downloader.s3_downloader import S3Downloader 
from file_transfer.infra.local_downloader.local_downloader import LocalDownloader 
from file_saver.infra.mongo_downloader.mongo_downloader import MongoDownloader


#rest
from handle_intro.infra.rest  import Rest



from handle_intro.infra.handle_intro.handle_intro import HandleIntro 

#Services
from handle_intro.services.handle_intro_service import HandleIntroService

#Controllers
from handle_intro.controllers.handle_intro_controller import  HandleIntroController

#Other Services
from handle_intro.services.text_utils_service import TextUtilsService
from handle_intro.services.speech_service import SpeechService
from handle_intro.services.time_service import AddTimeService

#from add_time.services.add_time_service import AddTimeService
#from add_time.infra.add_time import AddTime


from handle_intro.utils.helpers import *


class Factory(BaseFactory):
   
  
    def __init__(self, request: Request, logger = None):
        super().__init__(request, logger)                                      


    def make_script(self, script_name, general_config, script_config):
        return HandleIntro(script_config, self.logger)

    def make_external_service(self, script_name, general_config, script_config):
        
        service_config = {}
        services_config = get_attr(script_config, 'services', [])
        
        if (len(services_config)==0):
            raise Exception('Services config key is required')

        if (script_name == 'SPEECH-SERVICE'):
            
            service_config = list(filter(lambda service: service['type'] == 'speech',services_config))
            if (len(services_config) == 0):
                raise Exception('configuration for service speech is required')
            else:
                service_config = service_config[0]['config'] 
                script_config = { **script_config, **service_config }
                rest_executer = Rest(service_config, self.logger)
                return SpeechService(service_config, rest_executer)      

        elif (script_name == 'ADD-TIME-SERVICE'):    

            service_config = list(filter(lambda service: service['type'] == 'add-time',services_config))
            if (len(services_config) == 0):
                raise Exception('configuration for service add-time is required')
            else:
                service_config = service_config[0]['config']
                script_config = { **script_config, **service_config }
                #executer = AddTime(service_config, self.logger)
                executer = Rest(service_config, self.logger)
                downloader = script_config['downloader'] 
                #return AddTimeService(service_config, executer) 
                return AddTimeService(service_config, executer, downloader) 

        elif (script_name == 'TEXT-UTILS-SERVICE'):   

            service_config = list(filter(lambda service: service['type'] == 'text-utils',services_config))
            if (len(services_config) == 0):
                raise Exception('configuration for service text-utils is required')
            else:
                service_config = service_config[0]['config']  
                script_config = { **script_config, **service_config }
                rest_executer = Rest(service_config, self.logger)
                return TextUtilsService(service_config, rest_executer)

        else:
            raise Exception('Script name is not valid: ' + script_name)

        

        
    def make_saver(self, script_name, general_config, script_config):
        if (script_name == ''):
            if ('saver' not in script_config):
                raise Exception('saver configuration key is required')

            saver_type = script_config['saver']['type']

            script_config['saver'] = { **script_config['saver'], 'temp_directory': get_attr(script_config,'temp_directory','/tmp') }

            if (saver_type == 's3'):
                return S3FileSaver(script_config['saver'])   
            if (saver_type == 'local'):
                return LocalFileSaver(script_config['saver'])      
            elif (saver_type == 'download'):
                script_config['compress'] = True
                return PureFileSaver(script_config['saver'])    
            elif (saver_type == 'mongo'):
                return MongoUploader(script_config['saver'])              
            else:    
                raise Exception(f'Saver type not implemented: {saver_type}')    



    def make_downloader(self, script_name, general_config, script_config):
        

            if ('downloader' not in script_config):
                raise Exception('downloader configuration key is required')

            downloader_type = script_config['downloader']['type']

            script_config['downloader'] = { **script_config['downloader'], 'temp_directory': get_attr(script_config,'temp_directory','/tmp') }

            if (downloader_type == 's3'):
                return S3Downloader(script_config['downloader'])
            elif (downloader_type == 'local'):
                return LocalDownloader(script_config['downloader'])    
            elif (downloader_type == 'mongo'):
                return MongoDownloader(script_config['downloader'])      
            else:
                raise Exception(f'Downloader type not implemented: {downloader_type}')    

  
    def make_service(self, script_name, general_config, script_config) -> BaseService:
        
        script_config = script_config['config']

        saver = self.make_saver(script_name, general_config, script_config)
        downloader = self.make_downloader(script_name, general_config, script_config)
        executer = self.make_script(script_name, general_config, script_config)

        #adds external services into the main service
        external_services = []
        speech_service = self.make_external_service('SPEECH-SERVICE', general_config, script_config)
        time_service = self.make_external_service('ADD-TIME-SERVICE', general_config, script_config)
        text_service = self.make_external_service('TEXT-UTILS-SERVICE', general_config, script_config)

        external_services.append({'type': 'speech', 'instance': speech_service})
        external_services.append({'type': 'add-time', 'instance': time_service})
        external_services.append({'type': 'text-utils', 'instance': text_service})
       
        return HandleIntroService(script_config, executer, downloader, saver, external_services)

    def make_controller(self) -> BaseController:
        
        route = self.request.route
        script_name = self.get_script_name(route)
        service = self.make_service(script_name, self.general_config, self.script_config)
        
        controller = HandleIntroController(self.request, service)

        return controller     



        

    
    


class DefaultMessages:
    START = "Started script execution"
    END = "Finished script execution"
    

class Script:

    def __init__(self, script_name = 'unknown', logger = None, messages = None):
        self.messages = messages or DefaultMessages
        self.logger = logger 

    def add_message(self, message, type = 'info'):
        
        if (self.logger):
            if (type == 'info'):
                self.logger.info(message)
            elif (type == 'debug'):    
                self.logger.debug(message) 
            elif (type == 'warning'):    
                self.logger.warning(message)    
            elif (type == 'success'):    
                self.logger.success(message)          
            elif (type == 'error'):    
                self.logger.error(message)  
            else:
                self.logger.info(message)

        else:
            print(message)    

    def start(self):
        self.add_message(self.messages.START)
        
    def end(self):
        self.add_message(self.messages.END)


from abc import ABC
from handle_intro.controllers.base_controller import BaseController
from handle_intro.services.base_service import BaseService

from handle_intro.models.request import Request
from handle_intro.utils.helpers import *
import os

class BaseFactory(ABC):
   

    def get_from_body(self, payload, name):
        try:
            return payload[name]    
        except KeyError as err:
            raise Exception(f'Information required on body: {err}')
        except Exception as general_error:
            raise Exception(general_error) 

    def get_script_name(self, route):
        return route[1:]
  

    def __init__(self, request: Request, logger = None):

        self.request = request
        self.logger = logger

        body = self.request.body

        self.general_config = self.get_from_body(body, 'config')
        self.script_config = self.get_from_body(body, 'script_config')

        self.config_from_environment = self.get_config_from_environment()


    def get_config_from_environment(self):
       
        config = {
                'provider_config' : {
                    'client_id' : os.environ.get('PROVIDER_CLIENT_ID'),
                    'client_secret' : os.environ.get('PROVIDER_CLIENT_SECRET'),
                    'region' : os.environ.get('PROVIDER_REGION')
                }
        }

        return config
        


    def make_infra(self, script_name, general_config, script_config):
        pass

    def make_service(self, script_name, general_config, script_config) -> BaseService:
        pass

    def make_controller(self) -> BaseController:
        pass

  



        

    
    


from typing import AnyStr, List

class Author:
     def __init__(self, 
        name: AnyStr = None,
        link: AnyStr = None) -> None:

        self.name = name
        self.link = link

class Meta:
     def __init__(self, 
        description: AnyStr = None,
        duration: AnyStr = None,
        uploadDate: AnyStr = None,):

        self.description = description
        self.duration = duration
        self.uploadDate = uploadDate

class Image:
      def __init__(self, 
        title: AnyStr = None,
        meta: Meta = None,
        source: AnyStr = None,
        alternative_text: AnyStr = None):

        self.title = title
        self.meta = meta
        self.source = source
        self.alternative_text = alternative_text

class Movie:
      def __init__(self, 
        title: AnyStr = None,
        meta: Meta = None,
        source: AnyStr = None):

        self.title = title
        self.meta = meta
        self.source = source


class Section:
     def __init__(self, 
        title: AnyStr = None,
        description: AnyStr  = None,
        paragraphs: List[AnyStr] = None,
        images: List[Image] = None,
        movies: List[Movie] = None,
        section_type: AnyStr = None,
        related_id: AnyStr = None,
        _id: AnyStr = None,
        is_title = None,
        is_ad = None,
        attributes = None
        
        ) -> None:
    
        self.section_type = section_type
        self.id = _id
        self.title = title
        self.description = description,
        self.paragraphs = paragraphs
        self.images = images
        self.movies = movies
        self.attributes = attributes
        self.related_id = related_id
        self.is_ad = is_ad
        self.is_title = is_title
        

class Page:
    def __init__(self, 
     title = None,
     category = None,
     sections: List[Section] = None,
     summary: List[AnyStr] = None,
     authors: Author = None,
     publish_date: AnyStr = None,
     images: List[Image] = None,
     movies: List[Movie] = None
    ) -> None:
        
        self.title = title
        self.category = category
        self.sections = sections
        self.summary = summary
        self.authors = authors
        self.publish_date = publish_date
        self.images = images
        self.movies = movies

class Sentence:
     def __init__(self, 
                media_id = None,
                id = None,
                text = None,
                keywords = [],
                subtitles = [], 
                speech = None,
                time = 0) -> None:
        
        self.text = text
        self.keywords = keywords       
        self.subtitles =  subtitles
        self.speech = speech
        self.time = time
        self.media_id = media_id
        self.id = id
        


import requests
from requests_aws4auth import AWS4Auth

from handle_intro.utils.helpers  import get_attr



class AWSRequest:
    def __init__(self, config):

        aws_service = get_attr(config, 'aws_service', 'lambda')
        aws_region = get_attr(config, 'aws_region', 'us-east-1')
        aws_client_id = get_attr(config, 'aws_client_id', '')
        aws_client_secret = get_attr(config, 'aws_client_secret', '')
        endpoint = get_attr(config, 'endpoint', '')

        self.auth = AWS4Auth(aws_client_id, aws_client_secret, aws_region, aws_service)

     

  
    def request(self, endpoint, payload):
        response = requests.post(endpoint, data = payload, auth = self.auth)
        return response





from dotenv import load_dotenv
from os import environ 

from flask import Flask
from flask_restful import Resource, Api  #, reqparse

from handle_intro.handler import handler
from handle_intro.utils.lambda_helpers import get_json
from handle_intro.get_event import get_event

app = Flask(__name__)


API_PORT = environ.get('API_PORT')

class Api:
    
    @app.route('/', defaults={'path': ''}, methods = ['GET', 'POST'])
    @app.route('/<path:path>', methods = ['GET', 'POST'])
    def catch_all(path):
        
        load_dotenv()
        
        event = get_event()
        event = {**event, 'path': '/' + path}

        result = handler(event, None)
        json_str = get_json(result)
        return json_str
         

    def run(self):
        app.run(port=API_PORT)   


    

  








    

    
    



class MyVideo:
     def __init__(self, 
                media_id = None,
                keyword_id = None,
                section_id = None,
                url = None,
                original_url = None,
                path = None,
                path_converted = None, 
                thumb = None,
                time = 0,
                status = None,
                provider = None,
                provider_link = None,
                user = None,
                user_attribution = None,
                attribution = None) -> None:
        
        self.url = url
        self.original_url = original_url
        self.path = path       
        self.path_converted =  path_converted
        self.thumb = thumb
        self.time = time  
        self.status = status      
        self.provider = provider
        self.provider_link = provider_link
        self.user = user
        self.user_attribution = user_attribution
        self.media_id = media_id
        self.keyword_id = keyword_id
        self.section_id = section_id
        self.attribution = attribution




import os
import glob
import shutil
from urllib.parse import urlparse


import pickle
import json
import base64



def to_base64(data, decode = True):

    array_bytes = pickle.dumps(data)

    data = base64.b64encode(array_bytes) 
    if (decode == True):
        data = data.decode('ascii')

    return data


def read_binary_file(file):
   with open(file, 'rb') as f:
      return f.read()

def delete_directory(directory):
    shutil.rmtree(directory, ignore_errors=True, onerror=None)

def delete_file(file):
    os.remove(file)

def file_exists(file):
    return os.path.isfile(file)

def delete_all_files(directory):
    files = glob.glob(directory + '/*')
    for f in files:
        delete_file(f)

def get_all_files(directory):
    files = glob.glob(directory + '/*')
    return files


def copy_file(src, dst):
    shutil.copyfile(src, dst)        

def get_url_path(url):
    url_path = urlparse(url).path
    return url_path

def get_url_extension(url):
    url_path = get_url_path(url)
    ext = get_extension(url_path)
    return ext

def get_extension(file):
    try:
        
        ext = os.path.splitext(file)[1]
        ext = str(ext).replace('.', '')
        if (":" in str(ext)):
            ext = str(ext).split(':')[0]

        return ext 
    except:
        return ''    


def map_service_config(service_config, payload_key_name, payload):
    mapped_config = {**service_config, payload_key_name: payload}
    service_config = {"config": {}, "script_config": {"config": mapped_config }}
    return service_config


from handle_intro.services.base_service import BaseService
from handle_intro.utils.helpers import get_attr, set_attr
from handle_intro.utils.service_helper import map_service_config
from handle_intro.utils.file import to_base64


import base64
class AddTimeService(BaseService):

    def __init__(self, config, executer, downloader = None):

        self.config = config
        self.executer = executer
        self.downloader = downloader
        self.temp_directory =  get_attr(self.config,  'temp_directory', '/tmp')

    def get_time_to_sentence(self, sentence, binary_file = None):

        
        dict_sentence = sentence.__dict__
        
        payload = map_service_config(self.config,  "sentence", dict_sentence)

        #add configuration to downloader
        if (binary_file == None):
            if (self.downloader != None):
                set_attr(payload['script_config']['config'],'downloader', self.downloader )
        else: 
            base64_binary_file = to_base64(binary_file) 
            set_attr(payload['script_config']['config']['sentence']['speech'],'binary_file', base64_binary_file)

        result = self.executer.execute(payload = payload, service_path = '/sentence')
        

        return result 
        


from handle_intro.utils.helpers import get_attr
from handle_intro.infra.script import Script

from handle_intro.infra.aws_request import AWSRequest
import requests 
import json



SCRIPT_NAME="REST"




class Rest(Script):
   
    def __init__(self, config, logger = None):
        
        super().__init__(SCRIPT_NAME, logger)

        self.config = config
        
        self.on_lambda = get_attr(self.config, 'on_lambda', False)
        self.data = get_attr(self.config, 'data', {})
        self.temp_directory = get_attr(self.config, 'temp_directory', '/tmp')

        self.connection = get_attr(self.config, 'connection', {})

        self.base_url = get_attr(self.connection, 'base_url', '')
        self.method = get_attr(self.connection, 'method', 'POST')
        self.authentication = get_attr(self.connection, 'authentication', {})
        self.authentication_type = get_attr(self.authentication, 'type', '')

        
    def __get_headers(self):
        return self.authentication

    
    def execute(self, payload, service_path):

        result = {}

        #removes the path / in the end and in the begining  
        if(str(service_path).startswith('/')):
            service_path = service_path[1:len(service_path)]

        if(str(service_path).endswith('/')):
            service_path = service_path[0:len(service_path)-1]
        
        endpoint = self.base_url + '/' + service_path

        if (self.authentication_type == 'awsv4'):

            aws_requester = AWSRequest({ **self.authentication, "endpoint" : endpoint })
            json_payload = json.dumps(payload)
            response = aws_requester.request(endpoint = endpoint, payload = json_payload )
            if (response.ok == True):
                result = response.json()
            else:
                raise Exception(response.content)

            

        else:

            headers = self.__get_headers()

            if (self.method == 'POST'):
                response = requests.post(endpoint, data = payload, headers = headers)
                if (response.ok == True):
                    result = response.json()
                else:
                    raise Exception(response.content)


            if (self.method == 'GET'):
                response = requests.get(endpoint, data = payload, headers = headers)    
                if (response.ok == True):
                    result = response.json()
                else:
                    raise Exception(response.content)


        

        return result



from handle_intro.services.base_service import BaseService
from handle_intro.models.request import Request


class BaseController:

    def __init__(self, request: Request, service: BaseService):
        
        self.request = request
        self.service = service

        
    def handle(self):
        if (self.service):
            return self.service.execute()
  



        

    
    


import time

def is_valid_decimal(s):
        try:
            float(s)
        except ValueError:
            return False
        else:
            return True 

def decimal_to_time(value):
        if (str(value).isnumeric() or is_valid_decimal(value)):
            milisecs = str(round(float(value) % 1,3)).ljust(5,'0').replace('0.','')
            new_value = time.strftime('%H:%M:%S.' + str(milisecs), time.gmtime(value))
            return new_value
        else:
            return 0 


from handle_intro.utils.lambda_helpers import *

class Request:
    
    def get_param(self, params, name):
        try:
            return params[name]
        except KeyError as err:
            raise (f'Information required on path: {err}')
        except Exception as general_error:
            raise(general_error)     

    def get_from_body(self, payload, name):
        try:
            return payload[name]    
        except KeyError as err:
            raise (f'Information required on body: {err}')
        except Exception as general_error:
            raise(general_error) 

    def get_event_type(self, event):
        if ('Records' in event):
            if (len(event['Records']) > 0):
                if (len(event['Records']) == 1):
                    return event['Records']['eventSource']
                else:   
                    return 'invalid - more than one record'
            else:
                return 'invalid - no records'
        else:
            return 'direct'            
    
    def get_request_direct_call(self, event):
        body = from_body(event)
        route = get_route(event)
        params = get_path_params(event)
        header = from_header(event)
        user = get_user(event)
        return {'body': body, 'route': route, 'params': params, 'header': header, 'user': user}

    def get_request_s3_call(self, event):
        body = from_body(event)
        route = get_route(event)
        params = get_path_params(event)
        header = from_header(event)
        user = get_user(event)
        return {'body': body, 'route': route, 'params': params, 'header': header, 'user': user}

    def get_request_sqs_call(self, event):
        body = from_body(event)
        route = get_route(event)
        params = get_path_params(event)
        header = from_header(event)
        user = get_user(event)
        return {'body': body, 'route': route, 'params': params, 'header': header, 'user': user}

    def __init__(self, event):

        event_type = self.get_event_type(event)

        if (event_type == 'direct'):
            request = self.get_request_direct_call(event)
        elif  (event_type == 'aws:s3'):  
            request = self.get_request_s3_call(event)
        elif (event_type == 'aws:sqs'):   
            request = self.get_request_sqs_call(event)
        else:
            raise Exception(f'Invalid event - {event_type}')       

        self.route = request['route']
        self.params = request['params']
        self.body = request['body']
        self.header = request['header']
        self.user = request['user']


def read_from_file(file):
   with open(file) as f:
      return f.read()

def save_file(filename, content):
      f = open(filename, 'w')
      f.write(content)
      f.close()  
      return filename



from logger import Logger
from handle_intro.utils.lambda_helpers  import * 
from handle_intro.models.request import Request
from handle_intro.factories.factory import Factory

def handler(event, context):

    request = Request(event)
    logger = Logger()
    factory = Factory(request, logger)
    controller = factory.make_controller()
    result = controller.handle()

    json_str = get_json(result)
    
    print(json_str)
    
    return json_str


import json

def get_json(obj):
  return json.loads(
    json.dumps(obj, default=lambda o: getattr(o, '__dict__', str(o)))
  )

def get_user(event):
  if 'headers' in event:
    if 'user' in event['headers']:
      return event['headers']['user']
    else:
      return ''
  else:
    return event['requestContext']['authorizer']['iam']['userId']      
  
def from_body(event):
  body = event['body']
  if (type(body).__name__ == 'str'):
    body = json.loads(body)
  return body  

def get_path_params(event):
  if 'pathParameters' in event:
    return event['pathParameters']

def get_route(event):
  if 'path' in event:
    return event['path']  
  else:  
    return event['requestContext']['http']['path']

  
def from_header(event):
  return event['headers']


class Intro:
     def __init__(self, 
     media_id = None,
                path = None,
                path_converted = None,
                video_time = None,
                sentences = None) -> None:
        
        

        self.path = path
        self.path_converted = path_converted
        self.video_time = video_time
        self.media_id = media_id
        self.sentences = sentences

 


from dotenv import load_dotenv

from handle_intro.handler import handler
from handle_intro.utils.lambda_helpers import get_json
from handle_intro.get_event import get_event



class Application:
   
    def run(self):
        load_dotenv()
        event = get_event()
        result = handler(event, None)
        json_str = get_json(result)
        return json_str
           


    

  








    

    
    



from os import path
from handle_intro.services.base_service import BaseService
from handle_intro.utils.helpers import get_attr, generate_id, set_attr
from handle_intro.utils.json import  from_json
from handle_intro.utils.file import read_binary_file
from handle_intro.utils.video import get_video_duration
from handle_intro.models.page import Sentence
from handle_intro.models.intro import Intro


class HandleIntroService(BaseService):

            
    def replace_placeholders(self, text):
        new_text = text
        channel_name = get_attr(self.channel_info, 'name', '')
        new_text = str(new_text).replace("{{channel_name}}", f"{channel_name}")  
        return new_text   
    

    
    def get_service(self, external_services, type):
        services = list(filter(lambda service: service['type'] == type, external_services))
        if (len(services) == 0):
            raise Exception('external service of type ' + type + ' must be injected to the main service' )
        else:
            return services[0]['instance']    

    def __init__(self, config, executer, downloader, saver, external_services = []):

        self.config = config

        self.channel_info = get_attr(self.config, "channel_info", {})

        self.use_subtitles = get_attr(self.channel_info,  'use_subtitles', True)
        self.use_narration = get_attr(self.channel_info,  'use_narration', True)
        self.intro_muted = get_attr(self.channel_info,  'intro_muted', False)

        self.text = get_attr(self.config,  'text', '')

        #workers
        self.executer = executer
        self.downloader = downloader
        self.saver = saver

        self.speech_service = self.get_service(external_services, 'speech')
        self.time_service = self.get_service(external_services, 'add-time')
        self.text_utils_service = self.get_service(external_services, 'text-utils')

        self.temp_directory =   get_attr(self.config,  'temp_directory', '/tmp')
        self.ads =   get_attr(self.config,  'ads', [])

    
    def handle_intro(self, file_object):

        file_path = file_object['file']

        video_time = get_video_duration(file_path)

        #add sentence
        sentences = []
        if (len(str(self.text)) > 0):
            text = self.replace_placeholders(self.text)                
            sentence = Sentence(media_id = None, id = generate_id(),text=text,
                                keywords=None,subtitles=None,speech=None,time=video_time)
        

        #add keywords to sentence
        keywords = self.text_utils_service.extract_keywords(sentence)
        sentence.keywords = keywords

        #add subtitles to sentence
        if (self.use_subtitles == True ):
            subtitles = self.text_utils_service.generate_subtitles_for_sentence(sentence)
            sentence.subtitles = subtitles

        #add speech to sentence
        if ((self.use_narration == True )  and (self.intro_muted == False) ): 
            speech = self.speech_service.get_speech_for_sentence(sentence)
            sentence.speech = speech

        #temporary - for quick test    
        #sentence.speech = from_json('./model.json')


        #download files from s3/mongo when necessary
        binary_files = []
        download_required, downloaded_files = self.download_speech_files(sentence)
        if (download_required):
            
            #change the path of the file
            set_attr(sentence.speech, 'file', downloaded_files[0] )
            binary_files = self.read_binary_files(downloaded_files)

            #add times to sentence
            sentence =  self.time_service.get_time_to_sentence(sentence, binary_files[0])

        else:
            #add times to sentence
            sentence =  self.time_service.get_time_to_sentence(sentence)
 
        sentences.append(sentence)
        intro =  Intro(media_id = generate_id(), path = file_path, path_converted = '', video_time = video_time, sentences = sentences)
        return intro     

    def read_binary_files(self, files):
        binary_downloaded_files = []
        for file in files:
            binary_downloaded_files.append(read_binary_file(file))
        return binary_downloaded_files

    def download_speech_files(self, sentence):
        download_required = False
        downloaded_files = []
        

        #if speech exists and file is remote, download it before:
        speech = get_attr(sentence, 'speech', {})
        if (speech != None):
            source = get_attr(speech, 'source', {})
            if (source != None):
                source_bucket = get_attr(source, 'bucket_name',None)
                if (source_bucket != None): 
                    directory = path.dirname(source['file_path'])
                    file = source['file_id']
                    download_source = {'source_bucket' : source_bucket, "sources": [{ "directory": directory, "objects": [file] }]}
                    self.downloader.reset_sources()
                    self.downloader.add_sources(download_source)
                    download_results, downloaded_files = self.downloader.run()
                    download_required = True
                    if (len(downloaded_files) == 0):
                        raise Exception('download is required,but could not download files from s3 bucket')
                  

                else:
                    download_required = False
                    downloaded_files = [] 
            else:
                download_required = False
                downloaded_files = [] 

        else:
            download_required = False
            downloaded_files = []

        return (download_required, downloaded_files)      

    def execute(self):


        downloaded_files = []
        download_results = []
        result = None
        
        
        #download files
        download_results, downloaded_files = self.downloader.run()

        #save files into target files (for a specific project, for example)
        saved_files = self.saver.run(downloaded_files)     
        
        #convert files to thumbs
        for file in saved_files:
            result  = self.handle_intro(file)

        
        return result

        
       
      


from urllib.parse import urlparse

def get_domain(url):
    return urlparse(url).netloc
def get_full_domain(url):
    parsed_url = urlparse(url)
    return parsed_url[0] + '://' + parsed_url[1]
def is_absolute(url):
    return bool(urlparse(url).netloc)    


from handle_intro.services.base_service import BaseService
from handle_intro.utils.helpers import generate_id, get_attr
from  handle_intro.utils.service_helper import map_service_config

class TextUtilsService(BaseService):

    def __init__(self, config, executer):

        self.config = config
        self.executer = executer
        self.temp_directory =  get_attr(self.config,  'temp_directory', '/tmp')

   

    def extract_keywords(self, sentence):
        dict_sentence = sentence.__dict__
        payload = map_service_config(self.config,  "sentence", dict_sentence)
        result = self.executer.execute(payload = payload, service_path = '/extract-keywords-sentence')
        return result['keywords']   

    def generate_subtitles_for_sentence(self, sentence):
        dict_sentence = sentence.__dict__
        payload = map_service_config(self.config,  "sentence", dict_sentence)
        result = self.executer.execute(payload = payload, service_path = '/generate-subtitles-sentence')
        return result['subtitles']  



from mutagen.mp4 import MP4
from pymediainfo import MediaInfo


def get_video_duration(file):
    video = MP4(file)

    if (video.info.length > 0): 
        return video.info.length
    else:    
        media_info = MediaInfo.parse(file)
        duration_in_ms = media_info.tracks[0].duration
        return duration_in_ms / 1000




from handle_intro.services.base_service import BaseService

from handle_intro.models.request import Request
from handle_intro.controllers.base_controller import BaseController


class HandleIntroController(BaseController):

    def __init__(self, request: Request, service: BaseService):
        super().__init__(request, service)

    def handle(self):
        if (self.service):
            return self.service.execute()
  



        

    
    


class Ad:
     def __init__(self, 
                path = None,
                media_id = None,
                path_converted = None,
                media_type = None,
                time = None, 
                ad_video_time = None,
                attributions = None) -> None:
        
        

        self.path = path
        self.path_converted = path_converted
        self.time = time
        self.ad_video_time = ad_video_time
        self.media_id = media_id
        self.attributions = attributions
        self.media_type = media_type


import uuid
import random   


def generate_id():
    return str(uuid.uuid4())


def get_dict_attr(dict_object, attr_name, default_value = None):
 
    if attr_name in dict_object:
        return dict_object[attr_name]
    else:
        return default_value    

def get_random(min, max):
  return random.randint(min, max)

def set_attr(object, attr_name, value):
    if (type(object) is dict):
        #it is a dictionary
        if attr_name in object:
            object[attr_name] = value
            return object
        else:
            object[attr_name] = value
            return object
    else:
        #it is a python object
        if hasattr(object, attr_name):
            setattr(object, attr_name, value)
            return object
        else:
            return None
            

def get_attr(object, attr_name, default_value = None):

    if (type(object) is dict):
        #it is a dictionary
        if attr_name in object:
            return object[attr_name]
        else:
            return default_value    
    else:
        #it is a python object
        if hasattr(object, attr_name):
            return getattr(object, attr_name) 
        else:
            return default_value 

def get_value_by_attrs(object, attr_names, default_value = None):
    value = None
    for field in attr_names:
        value = get_attr(object, field, default_value)
        if (value != None):
            return value, field
    return value, field
    

def has_attr(object, attr_name):

    if (type(object) is dict):
        #it is a dictionary
        if attr_name in object:
            return True
        else:
            return False    
    else:
        #it is a python object
        if hasattr(object, attr_name):
            return True
        else:
            return False   

def has_attr_and_value(object, attr_name, reject_empty = False):

   has_att = has_attr(object, attr_name)
   if (has_att == True):
       value = get_attr(object, attr_name)
       if (reject_empty == False):
            return value!=None
       else:
           if (value == None):
               return False
           else:
               return (len(str(value)) > 0)

   else:
       return None    


def check_type(obj, _type):
    return isinstance(obj, _type)            

        



class SavedFile:
     def __init__(self, 
                file = None,
                status = None,
                message = None,
                target = None) -> None:

        self.file = file
        self.status = status
        self.message = message
        self.target = target